///////////////////////////////
#include <acknex.h>
#include <default.c>

#define STATE     skill1
#define ANIMATION skill2
///////////////////////////////

function main()
{
	level_load("shooter.wmb"); 	
}

function camera_follow(ENTITY* ent)
{
	while(1) 
	{
		vec_set(camera.x,vector(-150,10,50));  // camera position relative to the player      
		vec_rotate(camera.x,ent.pan); // rotate the camera position with the player
		vec_add(camera.x,ent.x);      // add player position
		vec_set(camera.pan,vector(ent.pan,-10,0)); // look in player direction, slighty down      
		wait(1);
	}
}


action spell_fly()
{
	my.ambient = 50;  // medium bright
	my.lightrange = 300; // activate dynamic light
	vec_set(my.blue,vector(255,50,50)); // bluish light color
	set(me,BRIGHT);   // additive blending

	vec_scale(my.scale_x,0.15); // small size
	c_setminmax(me);   // set my bounding box to my real size
	my.pan = your.pan; // face same direction as player
	my.STATE = 1;
	
	while(1)
	{
// state 1: flying ///////////////////////////////////////////  
		if (my.STATE == 1) 
		{
			my.roll += 20*time_step; 
			c_move(me,vector(40*time_step,0,0),NULL,IGNORE_YOU);
			if (HIT_TARGET)  // collided? 
				my.STATE = 2;  // go to next state
		}

// state 2: exploding ////////////////////////////////////////  
		if (my.STATE == 2) 
		{
			set(me,ZNEAR);  // appear in front of close objects
			my.roll = random(360);
			my.lightrange *= 1+0.5*time_step; // increase light range
			vec_scale(my.scale_x,1+0.5*time_step); // inflate size
			if (my.scale_x > 1) { // explosion finished? 
				ent_remove(me);
				return;
			}
		} 

		wait(1);	
	}
}

action wizard_walk()
{ 
	camera_follow(me);
	
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.STATE = 1;

	while (1)
	{
// state 1: walking ////////////////////////////////////////////
		if (my.STATE == 1)
		{
// rotate the entity with the arrow keys     
			my.pan += (key_cul-key_cur)*5*time_step;   

// move the entity forward/backward with the arrow keys
			var distance = (key_cuu-key_cud)*5*time_step;
			c_move(me, vector(distance,0,0), NULL, GLIDE);

// animate the entity    
			my.ANIMATION += 2*distance;
			ent_animate(me,"walk",my.ANIMATION,ANM_CYCLE);

// adjust entity to the ground height, using a downwards trace
			c_trace(my.x,vector(my.x,my.y,my.z-1000),IGNORE_ME | IGNORE_PASSABLE);
			my.z = hit.z - vFeet.z; // always place player's feet on the ground

			if (key_space) { // key pressed -> go to state 2
				my.ANIMATION = 0;
				my.STATE = 2;
			}
		}
		
// state 2: casting a spell /////////////////////////////////// 
		if (my.STATE == 2) 
		{
			my.ANIMATION += 8*time_step;
			ent_animate(me,"attack",my.ANIMATION,0);
			
			if (my.ANIMATION > 100) { // create the spell and go to state 3
				ent_create("spell.dds",vector(my.x,my.y,my.z+20),spell_fly);
				my.ANIMATION = 0;
				my.STATE = 3;
			}
		}  
		
// state 3: waiting for key release ///////////////////////////
		if (my.STATE == 3)
			if (!key_space) my.STATE = 1; // when key was released, back to state 1
		
		wait(1);
	}
}


function wizard_hit()
{
	my.STATE = 2;
}

action wizard_stand()
{
	my.event = wizard_hit;
	my.emask |= ENABLE_IMPACT;   
	my.STATE = 1;
	while (1)
	{  
// state 1: do nothing while waiting for impact //////////////// 
		
// state 2: death animation //////////////////////////////////// 
		if(my.STATE == 2) 
		{
			my.ANIMATION += 3*time_step; 
			ent_animate(me,"death",my.ANIMATION,0); // animate the entity
			if (my.ANIMATION > 70) 
				return;
		}

		wait(1);
	}
}

