///////////////////////////////
#include <acknex.h>
#include <default.c>

#define STATE     skill1
#define ANIMATION skill2
#define CREATOR   skill3

///////////////////////////////

var winner = 0;

function main()
{
	level_load("shooter.wmb"); 	

	while(1) {
		if (winner == 1) 
			draw_text("Well done! Red wins...",10,10,vector(50,50,255));
		else if (winner == 2) 
			draw_text("Ouch!! Green wins...",10,10,vector(50,255,50));
		wait(1);
	}
}

function camera_follow(ENTITY* ent)
{
	while(1) 
	{
		vec_set(camera.x,vector(-150,10,50));  // camera position relative to the player      
		vec_rotate(camera.x,ent.pan); // rotate the camera position with the player
		vec_add(camera.x,ent.x);      // add player position
		vec_set(camera.pan,vector(ent.pan,-10,0)); // look in player direction, slighty down      
		wait(1);
	}
}


action spell_fly()
{
	my.ambient = 50;  // medium bright
	my.lightrange = 300; // activate dynamic light
	vec_set(my.blue,vector(255,50,50)); // bluish light color
	set(me,BRIGHT);   // additive blending

	vec_scale(my.scale_x,0.15); // small size
	c_setminmax(me);   // set my bounding box to my real size
	my.pan = your.pan; // face same direction as player

// set FLAG2 to make the spell detectable by c_scan,
// and store the spell creator (you) in a skill
// so that the detecting entity gets its enemy pointer	
	set(my,FLAG2);     
	my.CREATOR = you;  
	
	my.STATE = 1;
	while(1)
	{
// state 1: flying ///////////////////////////////////////////  
		if (my.STATE == 1) 
		{
			my.roll += 20*time_step; 
			c_move(me,vector(40*time_step,0,0),NULL,IGNORE_YOU);
			if (HIT_TARGET)  // collided? 
				my.STATE = 2;  // go to next state
		}

// state 2: exploding ////////////////////////////////////////  
		if (my.STATE == 2) 
		{
			set(me,ZNEAR);  // render in front of close objects
			my.roll = random(360);
			my.lightrange *= 1+0.5*time_step; // increase light range
			vec_scale(my.scale_x,1+0.5*time_step); // inflate size
			if (my.scale_x > 1) { // explosion finished? 
				ent_remove(me);
				return;
			}
		} 

		wait(1);	
	}
}

// hit event
function wizard_hit()
{
	my.ANIMATION = 0;
	my.STATE = 4; // 4 = dead
}


action wizard_walk()
{ 
	camera_follow(me);

	my.event = wizard_hit;
	my.emask |= ENABLE_IMPACT;   

	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex

// set FLAG2 to make the wizard detectable by c_scan, 
// and store the wizard pointer in its own CREATOR skill 
// in case the wizard is detected directly
	set(my,FLAG2);  
  my.CREATOR = me;

	my.STATE = 1;
	while (1)
	{
// state 1: walking ////////////////////////////////////////////
		if (my.STATE == 1)
		{
// rotate the entity with the arrow keys     
			my.pan += (key_cul-key_cur)*5*time_step;   

// move the entity forward/backward with the arrow keys
			var distance = (key_cuu-key_cud)*5*time_step;
			c_move(me, vector(distance,0,0), NULL, GLIDE);

// animate the entity    
			my.ANIMATION += 2*distance;
			ent_animate(me,"walk",my.ANIMATION,ANM_CYCLE);

// adjust entity to the ground height, using a downwards trace
			c_trace(my.x,vector(my.x,my.y,my.z-1000),IGNORE_ME | IGNORE_PASSABLE);
			my.z = hit.z - vFeet.z; // always place player's feet on the ground

			if (key_space) { // key pressed -> go to state 2
				my.ANIMATION = 0;
				my.STATE = 2;
			}
		}
		
// state 2: casting a spell /////////////////////////////////// 
		if (my.STATE == 2) 
		{
			my.ANIMATION += 8*time_step;
			ent_animate(me,"attack",my.ANIMATION,0);
			
			if (my.ANIMATION > 100) { // create the spell and go to state 3
				ent_create("spell.dds",vector(my.x,my.y,my.z+20),spell_fly);
				my.ANIMATION = 0;
				my.STATE = 3;
			}
		}  
		
// state 3: waiting for key release ///////////////////////////
		if (my.STATE == 3) {
			if (!key_space) my.STATE = 1; // when key was released, back to state 1
		}
	
// state 4: death animation //////////////////////////////////// 
		if (my.STATE == 4) 
		{
			winner = 2; // other wizard won
			my.ANIMATION += 3*time_step; 
			ent_animate(me,"death",my.ANIMATION,0);
			if (my.ANIMATION > 70)
				return;
		}		
		
		wait(1);
	}
}


action wizard_stand()
{
	my.event = wizard_hit;
	my.emask |= ENABLE_IMPACT;   

  ENTITY* enemy = NULL; // spell target

	my.STATE = 1;
	while (1)
	{  
// state 1: wait until wizard or spell comes close enough ////// 
		if(my.STATE == 1) 
		{
// detect all FLAG2 entities within 750 quants 			
			c_scan(my.x,my.pan,vector(360,0,750),SCAN_ENTS | SCAN_FLAG2 | IGNORE_ME);
			if (you) {  // red wizard or flying spell detected?
				my.STATE = 2; 
				enemy = your.CREATOR;
			}
		}
	
// state 2: turn towards enemy, cast spells /////////////////// 
		if (my.STATE == 2) 
		{
// get the angle to the enemy			
			VECTOR vDirection;
			ANGLE vTargetAngle;
			vec_diff(vDirection,enemy.x,my.x);
			vec_to_angle(vTargetAngle,vDirection);
// vAngle is now the angle to the enemy.		
// Turn right or left depending on the difference
// between the current and the target pan angle			
			my.pan += time_step * sign(ang(vTargetAngle.pan - my.pan));
				
			my.ANIMATION += 3*time_step;
			ent_animate(me,"attack",my.ANIMATION,ANM_CYCLE);

			if (my.ANIMATION > 100) { // create the spell
				ent_create("spell.dds",vector(my.x,my.y,my.z+10),spell_fly);
				my.ANIMATION -= 100;    // continue the cycle from the beginning
			}

			if (enemy.STATE == 4) // enemy dead?
				my.STATE = 3;
		}  
		
// state 3: killed the enemy => dance of joy //////////////////// 
		if(my.STATE == 3) 
		{
			my.ANIMATION += 8*time_step; 
			ent_animate(me,"attack",my.ANIMATION,ANM_CYCLE);
		}
		
// state 4: death animation //////////////////////////////////// 
		if(my.STATE == 4) 
		{
			winner = 1; // other wizard (red) won
			my.ANIMATION += 3*time_step; 
			ent_animate(me,"death",my.ANIMATION,0); // animate the entity
			if (my.ANIMATION > 70) 
				return;
		}

		wait(1);
	}
}

