////////////////////////////////////////////////////////////////////////////
// simple lite-C online game
////////////////////////////////////////////////////////////////////////////
#include <acknex.h>
#include <default.c>

function on_client_event() // terminate client when server disconnects
{ 
   if (event_type == EVENT_LEAVE) sys_exit("Disconnected!"); 
}

function player_remove() // remove player when client disconnects
{ 
   if (event_type == EVENT_DISCONNECT) ent_remove(me); 
}

action player_move() // control the player on its client, move it on the server, animate it on all clients
{ 
  my.event = player_remove;
  my.emask |= ENABLE_DISCONNECT;
  my.smask |= NOSEND_FRAME; // don't send animation
   
  var walk_percentage = 0;
  var skill1_old = 0, skill2_old = 0;

  while (1) 
  {
    if (my.client_id == dplay_id)   // player created by this client?
    {
      if (key_w-key_s != my.skill1) { // forward/backward key state changed?
        my.skill1 = key_w-key_s; 
        send_skill(my.skill1,0); // send the key state in reliable mode
      }
      if (key_a-key_d != my.skill2) { // rotation key changed?
        my.skill2 = key_a-key_d; 
        send_skill(my.skill2,0); // send rotation state in reliable mode
      }
    }

    if (connection & CONNECT_SERVER) // running on the server?
    {
      if (my.skill1 != skill1_old) {    // if movement changed
        send_skill(my.skill1,SEND_ALL); // send movement to all clients
        skill1_old = my.skill1;
      }
      if (my.skill2 != skill2_old) {    // if rotation changed
        send_skill(my.skill2,SEND_ALL); // send rotation to all clients
        skill2_old = my.skill2;
      }
    }

    my.pan += my.skill2*5*time_step;   // rotate the entity using its skill2
    var distance = my.skill1*5*time_step;
    c_move(me, vector(distance,0,0), NULL, GLIDE); // move it using its skill1
    walk_percentage += distance;
    ent_animate(me,"walk",walk_percentage,ANM_CYCLE); // animate the entity

    wait (1);
  }
}

function main() 
{
  if (!connection) { // not started with -cl / -sv -cl?
    if (!session_connect(app_name,"")) // no client found on the localhost?
      session_open(app_name); // start as server
  }

  do { wait(1); }
  while (dplay_status < 2); // wait until the session is opened or joined
	
  dplay_entrate = 4;  // 16 ticks/4 = 4 updates per second
  dplay_smooth = 0;   // dead reckoning not needed
  dplay_localfunction = 2;
  level_load ("multiplayer6.wmb");
  vec_set(camera.x, vector (-600, 0, 100)); // set a proper camera position

  if (connection & CONNECT_SERVER) { // this instance of the game runs on the server
     STRING* title = str_create("Server: ");
     str_cat(title,server_ip);
     video_window(0,0,0,title);
     ent_create ("redguard.mdl",vector(100,50,40),player_move); // then create the red guard!
  } else { // otherwise, it runs on a connected client
     video_window(0,0,0,player_name);
     random_seed(0); // allow random player positions
     ent_create ("blueguard.mdl",vector(-100+random(200),-50+random(100),40),player_move); // create the blue guard
  }
}